@@ -50,9 +50,6 @@
 typedef std::function<bool(MainWindow&, QAction *)> checkfn;
 Q_DECLARE_METATYPE(checkfn)
 
-// TODO/FXIME: probably remove. QSS makes it unusable on mac...
-#define QSS_DROP    "MainWindow {border: 1px solid rgba(0, 0, 0, 50%);}\n"
-
 MainWindow::MainWindow(TerminalConfig &cfg,
                        bool dropMode,
                        QWidget * parent,
@@ -69,7 +66,8 @@ MainWindow::MainWindow(TerminalConfig &cfg,
       presetsMenu(nullptr),
       m_config(cfg),
       m_dropLockButton(nullptr),
-      m_dropMode(dropMode)
+      m_dropMode(dropMode),
+      m_layerWindow(nullptr)
 {
 #ifdef HAVE_QDBUS
     registerAdapter<WindowAdaptor, MainWindow>(this);
@@ -111,7 +109,6 @@ MainWindow::MainWindow(TerminalConfig &cfg,
     setContentsMargins(0, 0, 0, 0);
     if (m_dropMode) {
         this->enableDropMode();
-        setStyleSheet(QStringLiteral(QSS_DROP));
     }
     else {
         if (Properties::Instance()->saveSizeOnExit) {
@@ -176,17 +173,27 @@ void MainWindow::enableDropMode()
         winId();
         if (QWindow *win = windowHandle())
         {
-            if (LayerShellQt::Window* layershell = LayerShellQt::Window::get(win))
+            m_layerWindow = LayerShellQt::Window::get(win);
+            if (m_layerWindow)
             {
-                layershell->setLayer(LayerShellQt::Window::Layer::LayerOverlay);
-                layershell->setKeyboardInteractivity(LayerShellQt::Window::KeyboardInteractivityOnDemand);
-                LayerShellQt::Window::Anchors anchors = {LayerShellQt::Window::AnchorTop};
-                layershell->setAnchors(anchors);
+                m_layerWindow->setLayer(LayerShellQt::Window::Layer::LayerOverlay);
+                m_layerWindow->setKeyboardInteractivity(LayerShellQt::Window::KeyboardInteractivityOnDemand);
+                LayerShellQt::Window::Anchors anchors = {LayerShellQt::Window::AnchorTop
+                                                         | LayerShellQt::Window::AnchorBottom
+                                                         | LayerShellQt::Window::AnchorLeft
+                                                         | LayerShellQt::Window::AnchorRight};
+                m_layerWindow->setAnchors(anchors);
+                m_layerWindow->setScope(QStringLiteral("dropdown_terminal"));
+                m_layerWindow->setScreenConfiguration(LayerShellQt::Window::ScreenConfiguration::ScreenFromCompositor);
             }
         }
     }
 
     setWindowFlags(Qt::Dialog | Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint);
+    if (QGuiApplication::platformName() != QStringLiteral("wayland"))
+    { // some X11 WMs may need this
+        setWindowFlags(windowFlags() | Qt::Popup);
+    }
 
     m_dropLockButton = new QToolButton(this);
     m_dropLockButton->setToolTip(tr("Keep window open when it loses focus"));
@@ -215,13 +222,10 @@ void MainWindow::setDropShortcut(const QKeySequence& dropShortCut)
 void MainWindow::setup_Action(const char *name, QAction *action, const char *defaultShortcut, const QObject *receiver,
                               const char *slot, QMenu *menu, const QVariant &data)
 {
-    QSettings settings;
-    settings.beginGroup(QStringLiteral("Shortcuts"));
-
     QList<QKeySequence> shortcuts;
 
     actions[QLatin1String(name)] = action;
-    const auto sequences = settings.value(QLatin1String(name), QLatin1String(defaultShortcut)).toString().split(QLatin1Char('|'));
+    const auto sequences = Properties::Instance()->getShortcut(QLatin1String(name), QLatin1String(defaultShortcut)).split(QLatin1Char('|'));
     for (const QString &sequenceString : sequences)
         shortcuts.append(QKeySequence::fromString(sequenceString));
     actions[QLatin1String(name)]->setShortcuts(shortcuts);
@@ -299,7 +303,7 @@ void MainWindow::setup_ActionsMenu_Actions()
 
     data.setValue(checkSubterminals);
 
-    setup_Action(SUB_COLLAPSE, new QAction(tr("&Collapse Subterminal"), settingOwner),
+    setup_Action(SUB_COLLAPSE, new QAction(tr("&Close Subterminal"), settingOwner),
                  nullptr, consoleTabulator, SLOT(splitCollapse()), menu_Actions, data);
 
     setup_Action(SUB_TOP, new QAction(QIcon::fromTheme(QStringLiteral("go-up")), tr("&Top Subterminal"), settingOwner),
@@ -614,13 +618,44 @@ void MainWindow::toggleBookmarks()
     }
 }
 
+bool MainWindow::closePrompt(const QString &title, const QString &text)
+{
+    QDialog * dia = new QDialog(this);
+    dia->setObjectName(QStringLiteral("exitDialog"));
+    dia->setWindowTitle(title);
+
+    QCheckBox * dontAskCheck = new QCheckBox(tr("Do not ask again"), dia);
+    QDialogButtonBox * buttonBox = new QDialogButtonBox(QDialogButtonBox::Yes | QDialogButtonBox::No, Qt::Horizontal, dia);
+    buttonBox->button(QDialogButtonBox::Yes)->setDefault(true);
+
+    connect(buttonBox, &QDialogButtonBox::accepted, dia, &QDialog::accept);
+    connect(buttonBox, &QDialogButtonBox::rejected, dia, &QDialog::reject);
+
+    QVBoxLayout * lay = new QVBoxLayout();
+    lay->addWidget(new QLabel(QStringLiteral("<center>") + text + QStringLiteral("</center>")));
+    lay->addWidget(new QLabel(QStringLiteral("<center><i>") + tr("A process is running.") + QStringLiteral("</i></center>")));
+    lay->addStretch();
+    lay->addWidget(dontAskCheck);
+    lay->addWidget(buttonBox);
+    dia->setLayout(lay);
+
+    bool res(dia->exec() == QDialog::Accepted);
+    if (res)
+    {
+        Properties::Instance()->askOnExit = !dontAskCheck->isChecked();
+    }
+    dia->deleteLater();
+    return res;
+}
 
 void MainWindow::closeEvent(QCloseEvent *ev)
 {
     if (!Properties::Instance()->askOnExit
         || consoleTabulator->count() == 0
         // the session is ended explicitly (e.g., by ctrl-d); prompt doesn't make sense
-        || consoleTabulator->terminalHolder()->findChildren<TermWidget*>().count() == 0)
+        || consoleTabulator->terminalHolder()->findChildren<TermWidget*>().count() == 0
+        // there is no running process
+        || !consoleTabulator->hasRunningProcess())
     {
         disconnect(m_bookmarksDock, &QDockWidget::visibilityChanged,
                    this, &MainWindow::bookmarksDock_visibilityChanged); // prevent crash
@@ -645,30 +680,13 @@ void MainWindow::closeEvent(QCloseEvent *ev)
     }
 
     // ask user for cancel only when there is at least one terminal active in this window
-    QDialog * dia = new QDialog(this);
-    dia->setObjectName(QStringLiteral("exitDialog"));
-    dia->setWindowTitle(tr("Exit QTerminal"));
-
-    QCheckBox * dontAskCheck = new QCheckBox(tr("Do not ask again"), dia);
-    QDialogButtonBox * buttonBox = new QDialogButtonBox(QDialogButtonBox::Yes | QDialogButtonBox::No, Qt::Horizontal, dia);
-    buttonBox->button(QDialogButtonBox::Yes)->setDefault(true);
-
-    connect(buttonBox, &QDialogButtonBox::accepted, dia, &QDialog::accept);
-    connect(buttonBox, &QDialogButtonBox::rejected, dia, &QDialog::reject);
-
-    QVBoxLayout * lay = new QVBoxLayout();
-    lay->addWidget(new QLabel(tr("Are you sure you want to exit?")));
-    lay->addWidget(dontAskCheck);
-    lay->addWidget(buttonBox);
-    dia->setLayout(lay);
-
-    if (dia->exec() == QDialog::Accepted) {
+    if (closePrompt(tr("Exit QTerminal"), tr("Are you sure you want to exit?")))
+    {
         disconnect(m_bookmarksDock, &QDockWidget::visibilityChanged,
                    this, &MainWindow::bookmarksDock_visibilityChanged);
         Properties::Instance()->mainWindowPosition = pos();
         Properties::Instance()->mainWindowSize = size();
         Properties::Instance()->mainWindowState = saveState();
-        Properties::Instance()->askOnExit = !dontAskCheck->isChecked();
         Properties::Instance()->windowMaximized = isMaximized();
         rebuildActions();
         Properties::Instance()->saveSettings();
@@ -676,11 +694,11 @@ void MainWindow::closeEvent(QCloseEvent *ev)
             consoleTabulator->removeTab(i - 1);
         }
         ev->accept();
-    } else {
+    }
+    else
+    {
         ev->ignore();
     }
-
-    dia->deleteLater();
 }
 
 void MainWindow::actAbout_triggered()
@@ -689,7 +707,7 @@ void MainWindow::actAbout_triggered()
                      QStringLiteral("<center><b><big>QTerminal %1</big></b></center><br>").arg(qApp->applicationVersion())
                      + tr("A lightweight and powerful multiplatform terminal emulator")
                      + QStringLiteral("<br><br>")
-                     + tr("Copyright (C) ") + tr("2013-2022")
+                     + tr("Copyright (C) ") + tr("2013-2025")
                      + QStringLiteral("<br><a href='https://lxqt-project.org'>")
                      + tr("LXQt Project")
                      + QStringLiteral("</a><br><br>")
@@ -709,7 +727,6 @@ void MainWindow::propertiesChanged()
     rebuildActions();
 
     QApplication::setStyle(Properties::Instance()->guiStyle);
-    setWindowOpacity(1.0 - Properties::Instance()->appTransparency/100.0);
     consoleTabulator->setTabPosition((QTabWidget::TabPosition)Properties::Instance()->tabsPos);
     consoleTabulator->propertiesChanged();
     setDropShortcut(Properties::Instance()->dropShortCut);
@@ -757,9 +774,15 @@ void MainWindow::realign()
 {
     if (m_dropMode)
     {
+        if (m_layerWindow)
+        {
+            return; // done in showEvent
+        }
         QScreen *appScreen = QGuiApplication::screenAt(QCursor::pos());
         if(appScreen == nullptr)
+        {
             appScreen = QGuiApplication::primaryScreen();
+        }
         const QRect desktop = appScreen->availableGeometry();
         QRect g = QRect(desktop.x(),
                         desktop.y(),
@@ -768,7 +791,8 @@ void MainWindow::realign()
         g.moveCenter(desktop.center());
         // do not use 0 here - we need to calculate with potential panel on top
         g.moveTop(desktop.top());
-        if (g != geometry()) {
+        if (g != geometry())
+        {
             setGeometry(g);
         }
     }
@@ -864,13 +888,61 @@ bool MainWindow::event(QEvent *event)
     {
         if (m_dropMode &&
             !Properties::Instance()->dropKeepOpen &&
-            qApp->activeWindow() == nullptr
-            )
-            hide();
+            qApp->activeWindow() == nullptr)
+        {
+            // On Wayland and with a modal dialog, the dropdown window can be activated by
+            // clicking inside it and then deactivated by clicking on another window (see below).
+            if (!m_layerWindow || m_layerWindow->layer() == LayerShellQt::Window::Layer::LayerOverlay)
+            {
+                hide();
+            }
+        }
     }
+    // A workaround for the modal dialogs of the dropdown window on Wayland.
+    else if (event->type() == QEvent::WindowBlocked && m_layerWindow && m_dropMode)
+    {
+        if (auto dialog = qobject_cast<QDialog*>(qApp->activeModalWidget()))
+        {
+            dialog->winId();
+            if (QWindow *win = dialog->windowHandle())
+            {
+                if (LayerShellQt::Window *layershell = LayerShellQt::Window::get(win))
+                {
+                    layershell->setLayer(LayerShellQt::Window::Layer::LayerOverlay);
+                    layershell->setKeyboardInteractivity(LayerShellQt::Window::KeyboardInteractivityOnDemand);
+                    LayerShellQt::Window::Anchors anchors = {LayerShellQt::Window::AnchorTop};
+                    layershell->setAnchors(anchors);
+                    layershell->setScreenConfiguration(LayerShellQt::Window::ScreenConfiguration::ScreenFromCompositor);
+                    layershell->setScope(QStringLiteral("dialog"));
+                    // For preventing the dialog from going behind the dropdown window if
+                    // the latter is clicked, the window is set on the top layer temporarily.
+                    // It will be set on the overlay layer again when the dialog exits.
+                    m_layerWindow->setLayer(LayerShellQt::Window::Layer::LayerTop);
+                }
+            }
+        }
+    }
+    else if (event->type() == QEvent::WindowUnblocked && m_layerWindow && m_dropMode)
+    {
+        // Restore the overlay layer.
+        m_layerWindow->setLayer(LayerShellQt::Window::Layer::LayerOverlay);
+    }
+
     return QMainWindow::event(event);
 }
 
+void MainWindow::showEvent(QShowEvent* event)
+{
+    if (m_layerWindow && m_dropMode)
+    {
+        const QRect desktop = windowHandle()->screen()->availableGeometry();
+        int hMargin = desktop.width() * (100 - Properties::Instance()->dropWidth) / 200;
+        int vMargin = desktop.height() * (100 - Properties::Instance()->dropHeight) / 100;
+        m_layerWindow->setMargins(QMargins(hMargin, 0, hMargin, vMargin));
+    }
+    QMainWindow::showEvent(event);
+}
+
 void MainWindow::newTerminalWindow()
 {
     TerminalConfig cfg;
@@ -921,6 +993,13 @@ void MainWindow::bookmarksDock_visibilityChanged(bool visible)
 
 void MainWindow::addNewTab(TerminalConfig cfg)
 {
+    if (cfg.hasCommand())
+    {
+        // do not create subterminals if there is a command (-e option)
+        consoleTabulator->addNewTab(cfg);
+        return;
+    }
+
     if (Properties::Instance()->terminalsPreset == 3)
         consoleTabulator->preset4Terminals();
     else if (Properties::Instance()->terminalsPreset == 2)
